/**
 * See license.txt
 */
 
// constants
const FFE_TIME_NIGHT = -1; // time that night switches to day
const FFE_TIME_DAY = 15;  // time that day switches to night
const ffeIErrorItem = Components.interfaces.ffeIErrorItem;

//global forecastbar object
var gForecastbar = null;

function ffeLoad()
{
  gForecastbar = new forecastbar();
  gForecastbar.start();
}

function ffeUnload()
{
  gForecastbar.stop();
  gForecastbar = null;
}

///////////////////////
// forecastbar Object
function forecastbar() {}
forecastbar.prototype = {
  _manager: null,
  _bundle: null,
  _prefixes: null,
  
  start: function forecastbar_start()
  {      
    // migrate settings from Forecastfox and Forecastfox Enhanced
    this._enhancedMigrate();
    
    //get our XPCOM manager service
    var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    this._manager = Components.classes["@ensolis.com/forecastbar/manager-service;1"].getService(Components.interfaces.ffeIManagerService);
    this._bundle = document.getElementById("ffe-bundle");  
                  
    //setup object
    this._prefixes = {};
    this._prefixes["cc"] = "current";
    this._prefixes["dayf"] = "forecast";
    this._prefixes["dayt"] = "forecast";

    //add observer to forecastbar:update
    obs.addObserver(this, "forecastbar-manager", false);
    
    //add resize listener
    if (this._manager.canOverflow())
      window.addEventListener("resize", this, false);

    //setup display - give window a chance to display before loading
    var comp = this;
    if (!this._manager.isRunning)
      window.setTimeout(function() { comp._manager.run(false); }, 500);
    else
      window.setTimeout(function() { comp._update("display"); }, 500);
  },

  stop: function forecastbar_stop()
  {
    try {
      //remove observers
      var obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
      obs.removeObserver(this, "forecastbar-manager");
      obs = null;
      
      if (this._manager.canOverflow())
        window.removeEventListener("resize", this, false);
    } catch(e) {}

    //destroy variables
    this._manager = null;
    this._bundle = null;
    this._prefixes = null;
  },
  
  observe: function forecastbar_observe(aSubject, aTopic, aData)
  {
    var comp = this;
    window.setTimeout(function() { comp._update(aData); }, 0);
  },
  
  hideContext: function forecastbar_hideContext()
  {
    var node = document.getElementById("ffe-popup");    
    node.hidePopup();
  },
  
  addIcons: function forecastbar_addIcons(aEvent)
  {
    var icnSvc = this._manager.icons;    
    var node = aEvent.target;

    //remove old elements
    while (node.hasChildNodes() && node.lastChild.tagName != "menuseparator")
        node.removeChild(node.lastChild);
    
    //get icons
    var items = icnSvc.getItems({});
    var id = icnSvc.current.ID;
        
    //set none
    var item;    
    if (items.length == 0) {
      item = document.createElement("menuitem");
      item.setAttribute("label", this._bundle.getString("ff.icons.none"));
      item.setAttribute("disabled", true);
      node.appendChild(item);
      return;
    }
    
    //set for one or more
    for (var i=0; i<items.length; i++) {
      item = document.createElement("menuitem");
      item.setAttribute("label", items[i].name);
      item.setAttribute("value", items[i].ID);
      item.setAttribute("type", "radio");
      item.setAttribute("name", "icon");     
      if (items[i].ID == id)
        item.setAttribute("checked", "true");
      node.appendChild(item);
    }    
  },
    
  selectIcon: function forecastbar_selectIcon(aEvent)
  {
    var icnSvc = this._manager.icons;
      
    //set if not current
    var node = aEvent.target;
    var id = node.getAttribute("value");
    if (id != icnSvc.current.ID && id != "ffe-icons-more")
      icnSvc.current = icnSvc.getItem(id);

    node.parentNode.hidePopup();
  },
    
  addProfiles: function forecastbar_addProfiles(aEvent)
  {
    var prfSvc = this._manager.profiles;    
    var node = aEvent.target;

    //remove old elements
    while (node.hasChildNodes() && node.lastChild.tagName != "menuseparator")
        node.removeChild(node.lastChild);
        
    //get profiles
    var items = prfSvc.getItems({});
    var id = prfSvc.current.ID;
        
    //set none
    var item;    
    if (items.length == 0) {
      item = document.createElement("menuitem");
      item.setAttribute("label", this._bundle.getString("ff.profiles.none"));
      item.setAttribute("disabled", true);
      node.appendChild(item);
      return;
    }  
    
    //set for one or more
    for (var i=0; i<items.length; i++) {
      item = document.createElement("menuitem");
      item.setAttribute("label", items[i].name);
      item.setAttribute("type", "radio");
      item.setAttribute("name", "profile");
      item.setAttribute("value", items[i].ID);     
      if (items[i].ID == id)
        item.setAttribute("checked", "true");
      node.appendChild(item);
    }     
  },
  
  selectProfile: function forecastbar_selectProfile(aEvent)
  {
    var prfSvc = this._manager.profiles;
     
    //set if not current
    var node = aEvent.target;
    var id = node.getAttribute("value");
    if (id != prfSvc.current.ID)
      prfSvc.current = prfSvc.getItem(id);

    node.parentNode.hidePopup();
  },

  clickLink: function forecastbar_clickLink(aEl, event)
  {
    //middle clicked  
    if (event.button == 1) {
    
      //do oncommand
      eval(aEl.getAttribute("oncommand"));    
    }
  },
   
  openLink: function forecastbar_openLink(aEvent, aUnits, aPartner, aFrom)
  {
    // supply an aOpenEvent and it will be used in the openUILink instead of the aEvent
    var url = aEvent.target.getAttribute("url");
    if (url == "") {
      url = this._bundle.getString("ff.url.default");
      aPartner = true;
      aUnits = false;
    }
    
    var separator = (url.indexOf("?") != -1) ? "&" : "?";
    if (aPartner)
      url += separator + "partner=" + this._manager.partner;

    if (aUnits) {
      separator = (url.indexOf("?") != -1) ? "&" : "?";
 	    url += separator + "metric=" + this._manager.getPref("units.current");
    }

    this._manager.openLink(url, this._getWhere(aEvent, aFrom));
  },
  
  handleEvent: function forecastbar_handleEvent(aEvent)
  {
    this.updateOverflow(aEvent);
  },
  
  updateOverflow: function forecastbar_updateOverflow(aEvent)
  {
    //remove focus listener
    if (aEvent && aEvent.type == "focus")     
      window.removeEventListener("focus", this, false);
          
    //get elements  
    var chevron = document.getElementById("ffe-overflow-btn");
    var deck = document.getElementById("ffe-deck");
    var box = document.getElementById("ffe-box");
    var toolbar = box.parentNode;
    var popup = document.getElementById("ffe-overflow-popup");
    
    //add focus listener if toolbar is not shown
    if (toolbar.boxObject.width <= 0) {
      window.addEventListener("focus", this, false);
      return;
    }

    //ignore events that aren't on the document or the window
    if (aEvent && aEvent.target != document && 
        aEvent.target != window)
      return;
    
    //reset the display                  
    chevron.setAttribute("collapsed", "true");
    if (deck.parentNode != box) {
      deck.parentNode.removeChild(deck);
      box.appendChild(deck);
    }
    var orient = document.defaultView.getComputedStyle(toolbar, "").getPropertyValue("-moz-box-orient");
    this._setOrient(orient);
    
    //determine space needed for other elements
    var leftSize = 0;
    var neededSize = this._getElementSize(deck, orient);
    var items = document.getAnonymousNodes(toolbar, orient);
    if (!items)
      items = toolbar.childNodes; 
    for (var x=0; x<items.length; x++) {
      var item = items[x];
      if (item == box)
        continue;
          
      leftSize += this._getElementSize(item, orient);
    }
        
    /**
     * disable overflow until we can get it right
    var totalSize = (orient == "horizontal") ? window.innerWidth : window.innerHeight;
    if (neededSize > (totalSize - leftSize)) {
      this._setOrient("vertical");
      deck.parentNode.removeChild(deck);
      popup.appendChild(deck);
      chevron.removeAttribute("collapsed");
    } **/
  },
  
  importDOM: function forecastbar_importDOM()
  {
    var success = this._manager.migrator.importDOM(window);
    var prompter = this._getPrompter(window);
    if (!success) {
      var err = this._manager.migrator.lastError;
      if (err.severity < ffeIErrorItem.SEVERITY_ERROR)
        return;
      prompter.alert(err.name, err.message);
    } else
      prompter.alert(this._bundle.getString("ff.import.title"), 
                     this._bundle.getString("ff.import.success"));
  },
  
  exportDOM: function forecastbar_exportDOM()
  {
    var success = this._manager.migrator.exportDOM(window);
    var prompter = this._getPrompter(window);
    if (!success) {
      var err = this._manager.migrator.lastError;
      if (err.severity < ffeIErrorItem.SEVERITY_ERROR)
        return;
      prompter.alert(err.name, err.message);
    } else
      prompter.alert(this._bundle.getString("ff.export.title"), 
                     this._bundle.getString("ff.export.success"));
  },
  
  _getElementSize: function forecastbar__getElementSize(aElement, aOrient)
  {
    //setup variables
    var style = document.defaultView.getComputedStyle(aElement, "");
    var margin1 = 0;
    var margin2 = 0;
    var size = 0;
    
    //get width
    if (aOrient == "horizontal") {
      margin1 = style.getPropertyValue("margin-left");
      margin1 = margin1 ? Math.round(parseFloat(margin1)) : 0;
      margin2 = style.getPropertyValue("margin-right");
      margin2 = margin2 ? Math.round(parseFloat(margin2)) : 0;
      size = aElement.boxObject.width;
    
    //get heigh  
    } else {
      margin1 = style.getPropertyValue("margin-top");
      margin1 = margin1 ? Math.round(parseFloat(margin1)) : 0;
      margin2 = style.getPropertyValue("margin-bottom");
      margin2 = margin2 ? Math.round(parseFloat(margin2)) : 0;
      size = aElement.boxObject.height;
    }
    
    return size + margin1 + margin2;  
  },
  
  _setOrient: function forecastbar__setOrient(aOrient)
  {
    //element that need set
    var ids = ["ffe-box", "ffe-weather", "ffe-current-box", "ffe-spacer", "ffe-error",
      "ffe-forecast-box", "ffe-today-panel", "ffe-forecast-panel", "ffe-progress"];
      
    //set attribute
    for (var x=0; x<ids.length; x++)
      document.getElementById(ids[x]).setAttribute("orient", aOrient);
  },
    
  _getWhere: function forecastbar__getWhere(aEvent, aFrom)
  {
    //get left click default where
    var where = this._manager.getPref("links." + aFrom);
    
    //get link modifier
    try {
      var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
      var appBranch = pbs.getBranch(null);
      var background = appBranch.getBoolPref("browser.tabs.loadInBackground");
    } catch(e) {
      background = false;
    }
    
    //determine if middle click 
    var middle = (aEvent.button == 1);
    
    //get keys pressed
    var ctrl = aEvent.ctrlKey;
    var shift = aEvent.shiftKey;
    
    //do key based modifiers
    if (ctrl || middle) {
      if (shift || background)
        where = "tabshifted";
      else
        where = "tab";
    } else if (shift)
      where = "window";
          
    return where;
  },
   
  _moveBox: function forecastbar__moveBox()
  {
    //check if box is in the right location
    var toolbar = document.getElementById(this._manager.getPref("general.bar"));
    var box = document.getElementById("ffe-box");
    var position = this._manager.getPref("general.position");
    
    //if toolbar doesn't exist move to default location
    if (!toolbar) {
      this._manager.setPref("general.bar", "status-bar");
      this._manager.setPref("general.position", -1);
      return;
    }
    
    // bail if it is in the right place
    if (this._indexOf(toolbar,box) == position)
      return;

    //remove us from parent
    box.parentNode.removeChild(box);
    
    //make sure we have the right element type
    var newbox = null;
    if (toolbar.localName == "statusbar") {
      if (box.localName != "statusbarpanel")
        newbox = document.createElement("statusbarpanel");
    } else {
      if (box.localName == "statusbarpanel")
        newbox = document.createElement("hbox");    
    }
    
    //append children of old box
    if (newbox) {
      newbox.setAttribute("id", "ffe-box");
      while (box.hasChildNodes())
        newbox.appendChild(box.firstChild);      
    } else
      newbox = box;
    
    //insert us in correct place
    this._insertAtIndex(toolbar, newbox, position);
  },
  
  _indexOf: function forecastbar__indexOf(aParent, aChild)
  {
    // -1 if it does not exist
    var children = aParent.childNodes;
    for (var x=0; x<children.length; x++) {
      if (children[x] == aChild)
        return (x == children.length - 1) ? -1 : x;
    }
    return null;
  },
  
  _insertAtIndex: function forecastbar__insertAtIndex(aParent, aChild, aIndex)
  {
    var children = aParent.childNodes;
    if ((children.length == 0) || (aIndex >= children.length) || (aIndex < 0))
      aParent.appendChild(aChild);
    else
      aParent.insertBefore(aChild, children[aIndex]);
  },
  
  _update: function forecastbar__update(aMethod)
  {
    var err;
    
    //update all
    if (aMethod == "display") {
      this._update("startUpdate");
      this._update("endUpdate");
    }
    
    //check for an error
    if (aMethod == "endUpdate") {
      err = this._manager.lastError;
      if (err.severity != ffeIErrorItem.SEVERITY_INFO) {
        this._update("startUpdate");
        this._update("lastError");
        return;
      }
    }
    
    //move box
    this._moveBox();
    
    //remove progress and errors
    this._removeError();
    this._removeProgress();
    
    //now do update
    switch (aMethod) {
      case "startUpdate":
  	this._showProgress();
        break;
      case "lastError":
        this._showError();
        break;
      case "endUpdate":
      default:
        this._showCurrent();
        this._showForecast();
        break;
    }
    
    this._borders();
    
    //setup overflow
    this.updateOverflow();
  },

  _removeError: function forecastbar__removeError()
  {
    var item = document.getElementById("ffe-error"); 
    
    //remove error  
    item.removeAttribute("tooltiptext");
    item.removeAttribute("value");
    item.setAttribute("collapsed", true);
  },
  
  _removeProgress: function forecastbar__removeProgress()
  {
    var item = document.getElementById("ffe-progress");
    
    //remove progress
    item.removeAttribute("tooltiptext");
    item.setAttribute("mode", "determined");
    item.setAttribute("value", "0");
    item.setAttribute("collapsed", "true");
  },

  _showError: function forecastbar__showError()
  {
    var item = document.getElementById("ffe-error");   
    
    //set error
    var err = this._manager.lastError;
    item.setAttribute("value", err.name);
    item.setAttribute("tooltiptext", err.message);
    
    //show panel
    item.removeAttribute("collapsed");
    item.parentNode.setAttribute("selectedIndex", "1");      
  },
  
  _showProgress: function forecastbar__showProgress()
  {
    var item = document.getElementById("ffe-progress");   
    
    //set progress
    item.removeAttribute("value");
    item.setAttribute("mode", "undetermined");
    item.setAttribute("tooltiptext", this._bundle.getString("ff.tooltip.loading"));    
    
    if ( this._manager.getPref("update.showMeter") )
    {
    	//show panel
	item.removeAttribute("collapsed");
	item.parentNode.setAttribute("selectedIndex", "2");  
    }
  },

  _showCurrent: function forecastbar__showCurrent()
  {
    var current = this._getItem("current", "panel");
    var radar = this._getItem("radar", "panel");
    var swa = this._getItem("swa", "panel");
       
    //remove tooltip and attributes
    this._evictImages(current);
    this._evictImages(radar);
    this._evictImages(swa);
    current.removeAttribute("src");
    current.removeAttribute("label");
    radar.removeAttribute("src");
    radar.removeAttribute("label");
    swa.removeAttribute("src");
    swa.removeAttribute("label");
        
    //do current
    if (!this._manager.getPref("cc.panel.enabled"))
      current.setAttribute("collapsed", "true");
    else {
      this._setPanel(current, "cc", null, null);
      current.setAttribute("url", this._manager.parser.getValue("current", 0, "url", null));
      current.removeAttribute("collapsed"); 
    }
    
    //do radar
    if (!this._manager.getPref("radar.panel.enabled"))    
      radar.setAttribute("collapsed", "true");
    else {
      this._setPanel(radar, "radar", "radar", null);
      
      radar.setAttribute("url", this._manager.parser.getValue("radar", 0, "url", null));
      
      // only display the radar node if there is radar
      var image = this._manager.parser.getValue("radar", 0, "image", null);
      if (image && image != "N/A")
        radar.removeAttribute("collapsed");
      else
        radar.setAttribute("collapsed", true);
    }
    
    //do swa
    if (!this._manager.getPref("swa.panel.enabled") || this._manager.parser.getValue("swa", 0, "active", null) != "true")    
      swa.setAttribute("collapsed", "true");
    else {
      this._setPanel(swa, "swa", "swa", null);
      
      swa.setAttribute("url", this._manager.parser.getValue("swa", 0, "url", null));
      swa.removeAttribute("collapsed");       
    }
    
    //show panel
    var deck = document.getElementById("ffe-deck");
    deck.setAttribute("selectedIndex", "0");        
  },
  
  _showForecast: function forecastbar__showForecast()
  {
    var today = this._getItem("today", "panel");
    var extended = this._getItem("forecast", "panel");
       
    //remove tooltip and attributes
    while (today.hasChildNodes()) {
      this._evictImages(today.lastChild);
      today.lastChild.removeAttribute("src");
      today.lastChild.removeAttribute("label");
      today.removeChild(today.lastChild);
    }
    while (extended.hasChildNodes()) {
      this._evictImages(extended.lastChild);    
      extended.lastChild.removeAttribute("src");
      extended.lastChild.removeAttribute("label");
      extended.removeChild(extended.lastChild);
    }      
    
    //set url for popup menu since it is the same for all forecast days
    var menu = document.getElementById("ffe-popup-forecast");
    menu.setAttribute("url", this._manager.parser.getValue("days", 0, "url", null));
    
    //do today
    if (!this._manager.getPref("dayt.panel.enabled") || (this._manager.getPref("dayt.panel.mode") < 0))
      today.setAttribute("collapsed", "true");
    else
      this._updateToday(today);
  
    //do extended
    if (!this._manager.getPref("dayf.panel.enabled") || (this._manager.getPref("dayf.panel.days") == 0)) 
      extended.setAttribute("collapsed", "true");
    else
      this._updateExtended(extended);
    
    //show panel
    var deck = document.getElementById("ffe-deck");
    deck.setAttribute("selectedIndex", "0");        
  },
  
  _updateToday: function forecastbar__updateToday(aNode)
  {	
    var mode = this._manager.getPref("dayt.panel.mode");
    var sw_mode = this._manager.getPref("dayt.panel.switch");
    var panels = (mode == 2) ? 2 : 1;
    var cHours = this._getHour();
    var day_index = 0;
  	
    while (this._shouldIncDay(day_index, cHours))
      day_index++;
  	
    for (var x = 0; x < panels; x++) {
  	
      if (cHours <= FFE_TIME_NIGHT || cHours >= FFE_TIME_DAY) {
        if (mode == 0 && sw_mode == 1)
          day_index++;
        else if (mode == 2)
          panels--;
      }

      if (this._shouldCreatePanel("today", "days", x, cHours))
        this._createPanel("today", aNode, "days", day_index);
      else if (this._shouldCreatePanel("today", "nights", x, cHours))
        this._createPanel("today", aNode, "nights", day_index);
    }
  	
    // show the box!
    aNode.removeAttribute("collapsed");
  },
  
  _updateExtended: function forecastbar__updateExtended(aNode)
  {
	  var days = this._manager.getPref("dayf.panel.days");
    var new_panel = false;
    var cHours    = this._getHour();
    var day_index = 1;
    var day_add   = 0;

    while (this._shouldIncDay(day_add, cHours))
      day_add++;
      
    day_index += day_add;
        
    for (var x = 0; x < days; day_index++) {
      // finish if we went through too many days
      if (day_index > 8)
        break;
        
      try {
        // create the corresponding nodes
        if (this._shouldCreatePanel("extended", "days", day_index, cHours))
          new_panel = this._createPanel("extended", aNode, "days", day_index);
      } catch (e) {}
      
      try {
        if (this._shouldCreatePanel("extended", "nights", day_index, cHours))
          new_panel = this._createPanel("extended", aNode, "nights", day_index);
      } catch (e) {}
      
      // increase x if we made a new one
      if (new_panel)
        x++;
      
      new_panel = false;
    }
    
    aNode.removeAttribute("collapsed");
  },
  
  _shouldIncDay: function forecastbar__shouldIncDay(aIndex, aHours)
  {    
    return ((aHours < 12 && !this._hasData(aIndex)) && 
    		(FFE_TIME_NIGHT < aHours && aHours < FFE_TIME_DAY));
  },
  
  _hasData: function forecastbar__hasData(aIndex)
  {
    return ((this._manager.parser.getValue("days", aIndex, "t", null) != "N/A") ||
    	    (this._manager.parser.getValue("days", aIndex, "tlong", null) != "N/A"));
  },
  
  _shouldCreatePanel: function forecastbar__shouldCreatePanel(aType, aPart, aIndex, aHours) 	 
  {
    var mode, sw_mode, cHours, t_mode, t_sw_mode, t_on;    
    switch (aType) {
      case "today":
        mode = this._manager.getPref("dayt.panel.mode");
    	  sw_mode = this._manager.getPref("dayt.panel.switch");
    	
        switch (aPart) {
          case "days":
            if (aHours > FFE_TIME_NIGHT && aHours < FFE_TIME_DAY)
              return ((mode == 0) || (mode == 2 && aIndex == 0));
            else
              return ((mode == 0 && sw_mode == 1) || (mode == 2 && aIndex == 2));
          case "nights":
            if (mode == 1)
              return true;
            else if (aHours > FFE_TIME_NIGHT && aHours < FFE_TIME_DAY)
              return (mode == 2 && aIndex == 1);
            else
              return ((mode == 0 && sw_mode == 0) || (mode == 2));
        }
        break;
      case "extended":
        mode = this._manager.getPref("dayf.panel.mode");
        t_mode = this._manager.getPref("dayt.panel.mode");
        t_sw_mode = this._manager.getPref("dayt.panel.switch");
        t_on = this._manager.getPref("dayt.panel.enabled");

        switch (aPart) {
          
          case "nights":
            return ((mode == 1) || (mode == 2));
          case "days":
            // if we're displaying days or days and nights
            if (mode == 0 || mode == 2) {
              // If its between the "days" hours
              if (FFE_TIME_NIGHT < aHours && aHours < FFE_TIME_DAY)
                return true;
              // If its between the "nights" hours
              else
                return ((t_mode != 0) || (!t_on || (aIndex != 1 && t_sw_mode == 1)) || (t_sw_mode != 1));
            }
            else
              return false;
        }
        break;
    }
    
    return false;
  },
  
  _createPanel: function forecastbar__createPanel(aGroup, aParent, aPart, aDay)
  {
    var node = document.createElement("ffpanel");
    var prefix = (aGroup == "extended") ? "dayf" : "dayt";
    
    //set context menu on node
    node.setAttribute("context","ffe-popup");
    node.setAttribute("id", "forecast" + aPart + aDay);
    node.setAttribute("url", this._manager.parser.getValue(aPart, aDay, "url", null));

    //set panel and append
    this._setPanel(node, prefix, aPart, aDay);
    aParent.appendChild(node);
    
    // return that one was made
    return true;
  },
      
  _getItem: function forecastbar__getItem(aItem, aType)
  {
    var item = null;
    item = document.getElementById("ffe-" + aItem + "-" + aType);
    return item;  
  },
  
  _getHour: function forecastbar__getHour()
  {
    var time = this._manager.parser.getValue("global", 0, "tm", null);
    var ind = time.indexOf(":");
    if (ind < 0) {
      time = Number(this._manager.getPref("general.last"));
      return (new Date(time)).getHours();
    }
    return Number(time.substring(0,ind));
  },
  
  _setPanel: function forecastbar__setPanel(aNode, aPrefix, aGroup, aIndex)
  {
    //if group isn't provided get from prefix
    if (!aGroup)
      aGroup = this._prefixes[aPrefix];
      
    //set tooltip
    this._setTooltip(aNode, aPrefix, aGroup, aIndex);
       
    //set display  
    var icon = this._manager.parser.getValue(aGroup, aIndex, "icon", null);
    var label = this._manager.parser.getLabel(this._manager.getPref(aPrefix + ".panel.label"), aGroup, aIndex);
    icon = this._manager.icons.current.getItem(icon + "-small");
    
    switch (this._manager.getPref(aPrefix + ".panel.display")) {
      case 1:
        aNode.setAttribute("label", label);
        aNode.setAttribute("crop", "right");
        break;
      case 2:
        aNode.setAttribute("src", icon.URL);
        aNode.setAttribute("imagewidth", icon.width);
        aNode.setAttribute("imageheight", icon.height);
        aNode.setAttribute("label", label); 
        aNode.setAttribute("crop", "right");       
        break;
      case 0:
      default:
        aNode.setAttribute("src", icon.URL);
        aNode.setAttribute("imagewidth", icon.width);
        aNode.setAttribute("imageheight", icon.height);
        break;
    }
  },
  
  fillTooltip: function forecastbar_fillTooltip(aNode)
  {
    //do not show a tooltip if disabled
    var enabled = aNode.getAttribute("tooltipenabled");
    enabled = (enabled == "true");
    if (!enabled)
      return false;
      
    //get the tooltip attributes
    var title = aNode.getAttribute("tooltiptitle");
    var label = aNode.getAttribute("tooltiplabel");
    var display = aNode.getAttribute("tooltipdisplay");
    var src = aNode.getAttribute("tooltipsrc");
    var width = aNode.getAttribute("tooltipwidth");
    var height = aNode.getAttribute("tooltipheight");

    //get the tooltip element
    var tooltip = document.getElementById(aNode.getAttribute("tooltip"));
    
    //set the display
    tooltip.display = display;
    tooltip.title = title;
    tooltip.label = label;
    tooltip.src = src;
    tooltip.displayLabel(); 
    
    if ( aNode.getAttribute("id") == "ffe-radar-panel" )
    {
    	var radsize = this._manager.getPref("radar.size");

    	
    	switch (radsize)
    	{
    		case 0:
    			tooltip.imagewidth = 400;
    			tooltip.imageheight = 334;
    			break;
    		case 1:
    			tooltip.imagewidth = 640;
    			tooltip.imageheight = 480;
    			break;
    		case 2:
    			tooltip.removeAttribute("imagewidth");
    			tooltip.removeAttribute("imageheight");
    			break;
    		case 3:
    		default:
		    	tooltip.imagewidth = this._manager.getPref("radar.width");
		    	tooltip.imageheight = this._manager.getPref("radar.height");
		    	break;
	}
    }
    else {
    	tooltip.imagewidth = width;
    	tooltip.imageheight = height;
    }
      
    //show the tooltip
    return true;
  },
  
  _setTooltip: function forecastbar__setTooltip(aNode, aPrefix, aGroup, aIndex)
  {
    //if group isn't provided get from prefix
    if (!aGroup)
      aGroup = this._prefixes[aPrefix];
    
    //get the tooltip enabled and display prefs
    var enabled = this._manager.getPref(aPrefix + ".tooltip.enabled");
    var display = this._manager.getPref(aPrefix + ".tooltip.display");
            
    //get icon attributes
    var icon = this._manager.parser.getValue(aGroup, aIndex, "icon", null);
    icon = this._manager.icons.current.getItem(icon + "-large");
    var src = icon.URL;
    var width = icon.width;
    var height = icon.height;
    
    //get the label attributes
    var label = this._manager.getPref(aPrefix + ".tooltip.label");    
    label = this._manager.parser.getLabel(label, aGroup, aIndex);
    var title = this._manager.getPref(aPrefix + ".tooltip.title"); 
    title = this._manager.parser.getLabel(title, aGroup, aIndex);    

    //set the attributes on the node
    aNode.setAttribute("tooltipenabled", String(enabled));
    aNode.setAttribute("tooltipdisplay", String(display));
    aNode.setAttribute("tooltipsrc", String(src));
    aNode.setAttribute("tooltipwidth", String(width));
    aNode.setAttribute("tooltipheight", String(height));
    aNode.setAttribute("tooltiptitle", title);
    aNode.setAttribute("tooltiplabel", label);
    aNode.setAttribute("tooltip", "ffe-tooltip");
  },
  
  _borders: function forecastbar__borders()
  {
    //get elements
    var spacer = document.getElementById("ffe-spacer");
    var today = this._getItem("today", "panel");
    var extended = this._getItem("forecast", "panel");
    var radar = this._getItem("radar", "panel");
    var swa = this._getItem("swa", "panel");
    var current = this._getItem("current", "panel");
    var deck = document.getElementById("ffe-deck");
    
    var showing = new Object();
    
    //determine what is showing
    showing["deck"] = (deck.getAttribute("selectedIndex") != "0") ? true : false;
    showing["current"] = (this._manager.getPref("cc.panel.enabled") && !current.hasAttribute("collapsed"));
    showing["radar"] = (this._manager.getPref("radar.panel.enabled") && !radar.hasAttribute("collapsed"));
    showing["swa"] = (this._manager.getPref("swa.panel.enabled") && !swa.hasAttribute("collapsed"));
    showing["today"] = (this._manager.getPref("dayt.panel.enabled") && today.hasChildNodes() && !today.hasAttribute("collapsed"));
    showing["extended"] = (this._manager.getPref("dayf.panel.enabled") && extended.hasChildNodes() && !extended.hasAttribute("collapsed"));
    
    //show-hide spacer
    var cShowing = (showing["deck"] || showing["current"] || showing["radar"] || showing["swa"]);
    var fShowing = (showing["deck"] || showing["today"] || showing["extended"]);
    if (cShowing && fShowing)
      spacer.removeAttribute("collapsed");
    else
      spacer.setAttribute("collapsed", "true");
        
    //remove old first-last attribute
    var box = document.getElementById("ffe-box");
    var panels = box.getElementsByTagName("ffpanel");
    for (var x=0; x<panels.length; x++) {
      panels[x].removeAttribute("ffe-first");
      panels[x].removeAttribute("ffe-last");
    }
    
    if (!showing["deck"]) {
      //set first attribute
      if (showing["swa"])
        swa.setAttribute("ffe-first", "true");    
      else if (showing["radar"])
        radar.setAttribute("ffe-first", "true");
      else if (showing["current"])
        current.setAttribute("ffe-first", "true");
      else if (showing["today"])
        today.firstChild.setAttribute("ffe-first", "true");
      else if (showing["extended"])
        extended.firstChild.setAttribute("ffe-first", "true");

      //set last attribute
      if (showing["extended"])
        extended.lastChild.setAttribute("ffe-last", "true");
      else if (showing["today"])
        today.lastChild.setAttribute("ffe-last", "true");
      else if (showing["current"])
        current.setAttribute("ffe-last", "true");
      else if (showing["radar"])
        radar.setAttribute("ffe-last", "true");
      else if (showing["swa"])
        swa.setAttribute("ffe-last", "true");
    }
  },
  
  _evictImages: function forecastbar__evictImages(aNode)
  {
    //remove the source image
    var image = aNode.getAttribute("src");
    if (image != "")
      this._manager.evictImage(image);
      
    //remove the tooltip source image
    image = aNode.getAttribute("tooltipsrc");
    if (image != "")
      this._manager.evictImage(image);
  },
  
  _getPrompter: function forecastbar__getPrompter(aParent)
  {
    //get the watcher service
    var watcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].
                  getService(Components.interfaces.nsIWindowWatcher);
                
    //return a prompter
    return watcher.getNewPrompter(aParent);
  },
  
  openDialog: function forecastbar_openDialog(aType)
  {
    //focus the window if already open
    var mediator = Components.classes['@mozilla.org/appshell/window-mediator;1'].
                   getService(Components.interfaces.nsIWindowMediator);
    var win = mediator.getMostRecentWindow(aType);
    if (win) {
      win.focus();
      return;
    }
      
    //open a new window       
    switch (aType) {
    case "forecastbar:options":
      window.openDialog("chrome://forecastbar/content/options.xul", "options", "chrome");
      break;
      
    case "forecastbar:about":
      window.openDialog("chrome://forecastbar/content/about.xul", "ffe-about", "chrome");
      break;
    default:
      break;
    }
   },
 
   _pauseUpdates: function()
   {
      var updatesPaused = this._manager.getPref("update.paused");
 
      if (updatesPaused)
      {
         this._manager.setPref("update.paused", false);
       }
      else
      {
         this._manager.setPref("update.paused", true);
       }
 
      return;
    },
 
    _setUpdatePause: function()
   {
      //check if updates are paused
      var updatesPaused = this._manager.getPref("update.paused");
      document.getElementById("ffe-disable-updates").setAttribute("checked", updatesPaused);
 
      return true;
   },
 
    _forceLoad: function()
    {
       //allow manual reload even if updates are paused
       this._manager.setPref("update.force", true);
       this._manager.run(true);
 
       return;
    },
    
    _enhancedMigrate: function()
    {
       //import settings from Forecastfox and Forecastfox Enhanced
       var pbSvc = Cc["@mozilla.org/preferences-service;1"].
              getService(Ci.nsIPrefService);
       var dirSvc = Cc["@mozilla.org/file/directory_service;1"].
                  getService(Ci.nsIProperties);
       var branch = pbSvc.getBranch("forecastbar.");
       if (! branch.prefHasUserValue("fcbe.migrated")) {
          var ffdir = dirSvc.get("ProfD", Ci.nsIFile);
          ffdir.append("forecastfox");
          if (ffdir.exists()) {
           // copy files
       	   try {
              var profdir = dirSvc.get("ProfD", Ci.nsIFile);
              var ffedir = profdir.clone();
              ffedir.append("forecastbar");
              if (ffedir.exists())
                 ffedir.remove(true);
              ffdir.copyToFollowingLinks(profdir, "forecastbar");
      	   } catch (e) {}
      	   
      	   //copy preferences
           var ffbranch = pbSvc.getBranch("forecastfox.");
           var children = ffbranch.getChildList("",{});
           for( var i in children ) {
                   var rv = "";
	  	   switch (ffbranch.getPrefType(children[i])) {
	  	     case Ci.nsIPrefBranch.PREF_INT:
	  	       rv = ffbranch.getIntPref(children[i]);
		       branch.setIntPref(children[i], rv);
		       break;
		     case Ci.nsIPrefBranch.PREF_BOOL:
		       rv = ffbranch.getBoolPref(children[i]);
		       branch.setBoolPref(children[i], rv);
		       break;
		     case Ci.nsIPrefBranch.PREF_STRING:
		     default:
		       try {    
                         var plString = ffbranch.getComplexValue(children[i], Ci.nsIPrefLocalizedString).data;
                         branch.setComplexValue(children[i], Ci.nsIPrefLocalizedString, plString);
			                          
		       } catch(e) {
			 try {   
        		   var sString = ffbranch.getComplexValue(children[i], Ci.nsISupportsString);
        		   branch.setComplexValue(children[i], Ci.nsISupportsString, sString);
			 } catch(e) {
			   branch.setCharPref(children[i], ffbranch.getCharPref(children[i])); 
			 }
		       }
		       break;    
		   }
	   }
      	}
        branch.setCharPref("fcbe.migrated", "0.9.6");
     }
   }  
};
