/**
 * See license.txt
 */

/******************************************************************************
 * Interfaces used monitor conflicting installations of
 * forecastbar and forecastbar Enhanced
 * 
 * @status    Development
 * @version   1.0
 ******************************************************************************/
function ConflictService() 
{
  //setup additional interfaces
  this._ifaces.push(Ci.nsIObserver);
  this._ifaces.push(Ci.nsISupportsWeakReference);
  this._ifaces.push(Ci.nsIRDFObserver);

  //setup a new error
  this._error = Cc["@ensolis.com/forecastbar/error-item;1"].
                createInstance(Ci.ffeIErrorItem);        
}
ConflictService.prototype = {
  __proto__: new ServiceBase("ConflictService"),
  _install: null,
  _obSvc: null,
  _dtaSrc: null,
  _exMgr: null,
  
  ////////////////////////////
  // nsIObserver 
  observe: function ConflictService_observe(aSubject, aTopic, aData) 
  {
    //only do something if item is forecastbar
    var item = aSubject.QueryInterface(Ci.nsIUpdateItem);
    if (item.id != "{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}")
      return;
    
    
    //determine action based on message  
    switch (aData) {
        
     //being installed or enabled
    case "item-installed":
    case "item-enabled":
       getTopWindow().openDialog("chrome://forecastbar/content/conflict.xul", "ff-conflict-window", "chrome,centerscreen=yes");
      break;
      
        
    //being cancelled
    case "item-cancel-action":
      this.run();
      break;
    }
  },
   

                   
  ////////////////////////////////
  // ffeIService  
  
  /**
   * Initialize the component.  Called by the manager service.
   */  
  start: function ConflictService_start()
  {
    //set install flag
    this._install = false;
                    
    //get extension manager - toolkit 1.0
    if ("@mozilla.org/extensions/manager;1" in Cc) {
      this._exMgr = Cc["@mozilla.org/extensions/manager;1"].
                  getService(Ci.nsIExtensionManager);
             
      //manager supports observers - toolkit 1.5 or greater
      if ("getItemForID" in this._exMgr) {
        this._obSvc = Cc["@mozilla.org/observer-service;1"].
                    getService(Ci.nsIObserverService);        
        this._obSvc.addObserver(this, "em-action-requested", false);
      
      //manager doesn't support observers - toolkit 1.0
      } else {
          this._dtaSrc = this._exMgr.datasource;
          this._dtaSrc.AddObserver(this);
      }
    }
    
    //return success
    return true;
  },
  
  /**
   * Destroy the component.  Called by the manager service.  This may be
   * called prior to start so it needs to be safe.
   */  
  stop: function ConflictService_stop()
  {
    //unregister from the observer service
    if (this._obSvc) 
      this._obSvc.removeObserver(this, "em-action-requested");
      
    //unregister as an observer of the em datasource  
    if (this._dtaSrc)
      this._dtaSrc.RemoveObserver(this);
    
    //remove variables
    this._install = null;
    this._obSvc = null;
    this._dtaSrc = null;
    this._exMgr = null;
  },
                
  ////////////////////////////////
  // ffeIConflictService  
    
  /**
   * Run conflict resolution.
   */  
  run: function ConflictService_run()
  {
    var nsIUpdateItem = Components.interfaces.nsIUpdateItem;
    var itemType = nsIUpdateItem.TYPE_EXTENSION;
              
    items = this._exMgr.getItemList(itemType, { });

    for (var i in items) {
      if (items[i].id == "{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}") {
        getTopWindow().openDialog("chrome://forecastbar/content/conflict.xul", "ff-conflict-window", "chrome,centerscreen=yes");
      }
    }
    return;
  },
  
  ////////////////////////////
  // Internal Functions 
  
  /**
   * Datasource observer function.  Determines the action to run
   * based on datasource updates.
   *
   * @param   The Datasource being updated.
   * @param   The subject being updated.
   * @param   The predicate being updated.
   * @param   The object being updated.
   */
  _dsObserve: function ConflictService__dsObserve(aDS, aSource, aProperty, aTarget)
  {
    //only do something if item is forecastbar      
    if (aSource.ValueUTF8 != "urn:mozilla:extension:" + "{0538E3E3-7E9B-4d49-8831-A227C80A7AD3}")
      return;
    
    //get the migrator service
    var mgrSvc = Cc["@ensolis.com/forecastbar/manager-service;1"].
                 getService(Ci.ffeIManagerService);
    var migSvc = mgrSvc.migrator;
    
    //determine action based on predicate  
    var prefix = "http://www.mozilla.org/2004/em-rdf#";    
    var action = aProperty.ValueUTF8.replace(prefix, "");
    switch (action) {
    
    //being installed or enabled
    case "toBeInstalled":
    case "toBeEnabled":
      this._install = true;
      break;
      
    default:
      break;
    }
  }
};
